#include "lpc2103.h"
#include "io.h"
#include "sin.h"

//for the proto with the microphone amp connected to ad2, enable this. That's you,
// Maarten!
//#define USE_AD2

//I/O-pins
#define LCD_RESET	(8)
#define LCD_CS		(7)
#define LCD_CLK		(4)
#define LCD_DAT		(6)
#define LCD_BL		(9)
#define NAND_WP		(21)
#define NAND_WE		(20)
#define NAND_ALE	(19)
#define NAND_CLE	(18)
#define NAND_CE		(17)
#define NAND_RE		(16)
#define NAND_RB		(15)
#define NAND_IO7	(13)
#define NAND_IO6	(26)
#define NAND_IO5	(25)
#define NAND_IO4	(12)
#define NAND_IO3	(11)
#define NAND_IO2	(10)
#define NAND_IO1	(24)
#define NAND_IO0	(23)

//Enable this to use the software bitbang algorithm to send data to the LCD
//instead of the more speedy hardware SPI algorithm.
//#define BITBANG

void processor_init() {
    //Set memmap for running from rom
    MEMMAP=1;
    //Enable mam, cycles of 3 CCLKs.
    MAMCR=0;
    MAMTIM=3;
    MAMCR=2;
    //Turn on PLL
    PLLCFG=2+(2<<5); //msel+(psel<<5), this enables a 60MHz clock
    PLLCON=1; //enable PLL
    PLLFEED=0xAA; PLLFEED=0x55;
    while ((PLLSTAT&(1<<10))==0); //wait for pll lock
    PLLCON=3; //connect PLL
    PLLFEED=0xAA; PLLFEED=0x55;
    //APB divider = 0
    APBDIV=0x01;
}

//Fuzzy-logic waiting routines ;-)
void waitsome(void) { //wait 'a certain amount'
    volatile int x;
    for (x=0; x<153; x++);
}

void waitabit(void) { //wait 'a bit less'
    volatile int x;
    for (x=0; x<53; x++);
}

//Set up I/O
void io_init() {
    //Set up special pins: spc clk + mosi
#ifdef BITBANG
    PINSEL0=0;
#else
    PINSEL0=0x00001105;
    S0SPCR=0x93C; //setup SPC
    S0SPCCR=0x8; //SPC clk divider
#endif

#ifdef USE_AD2
    PINSEL1=0x0000c000;
#else 
    PINSEL1=0x00003000;
#endif
    SCS=1; //hi-speed gpio enable
    FIODIR=(1<<LCD_RESET)|(1<<LCD_CLK)|(1<<LCD_BL)|(1<<LCD_CS)|(1<<LCD_DAT);
    //Init a/d
#ifdef USE_AD2
    AD0CR=0x020FF02; 
#else 
    AD0CR=0x020FF01;
#endif
    //reset LCD
    FIOSET=(1<<LCD_CS);
    FIOCLR=(1<<LCD_RESET);
    waitsome();
    FIOSET=(1<<LCD_RESET);
    waitsome();
    FIOSET=(1<<LCD_BL);
}

//Send a byte to the LCD. Add 0x100 to the byte to indicate it's a command.
void sendlcd(int d) {
#ifdef BITBANG
    int x;
#endif
    if (d&0x100) {
	FIOSET=(1<<LCD_CS);
	FIOCLR=(1<<LCD_CS);
    }
    d^=0x100; //it's /cmd, not positive cmd as the rest of the code thinks
    
#ifdef BITBANG
    for (x=0; x<9; x++) {
	if (d&0x100) {
	    FIOSET=(1<<LCD_DAT);
	} else {
	    FIOCLR=(1<<LCD_DAT);
	}
	FIOCLR=(1<<LCD_CLK);
	waitabit();
	FIOSET=(1<<LCD_CLK);
	waitabit();
	d=d<<1;
    }
    waitabit();
    waitabit();
#else
    S0SPDR=d;
    while(!(S0SPSR&(1<<7))) ;
//    waitabit();
#endif
}


static unsigned short avg;

//This routine waits for the preamp logic to bias at a certain average value.
void calibratead() {
    int x;
    unsigned int a, oa,d;
    
    avg=0;
    a=0;
    do {
	oa=a;
	for (x=0; x<1024; x++) {
	    a+=getad();
	}
	a/=1024;
	d=oa-a;
	if (d<0) d=-d;
    } while (d>100);
    avg=a;
}

//Let the A/D read a value and return it
signed short getad() {
    AD0CR|=0x1000000;
    while(!(AD0GDR&(1<<31)));
    return((AD0GDR>>6)&0x3ff)-avg;
}

