#include "io.h"
#include "lcd.h"
#include "sin.h"

#define INVERTED_LCD

unsigned char dispram[6566];
unsigned short pal[256];

//Init the LCD
void lcd_init() {
    //This init sequence is needed for the LCD to initialize into a 12-bit 
    //color mode. -1 means 'wait some time', the sequence ends with -2.
    const short sendus[]={
	0x101, -1, 0x1c6, -1, 0x1b9, 0, 0x11b6, 0x80, 0x80, 0x81,
	84, 69, 82, 67, 0x1b3, 0, 8, 17, 25, 34, 42, 51, 60, 68, 77,
	85, 94, 103, 112, 127, 0x1b5, 1, 0x1bd, 4, 0x1be, 4, 0x111, -1,
	0x1ba, 0xff, 0x3, 0x1b7, 0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0x103, -1,
	0x120, 0x129, -2};

    int x=0;

    while (sendus[x]!=-2) {
	if (sendus[x]==-1) {
	    waitsome();
	    waitsome();
	    waitsome();
	    waitsome();
	} else {
	    sendlcd(sendus[x]);	
	}
	x++;
    }
//Original init sequence: uglier & taking up more flash.
/*
    sendlcd(0x101); //sw reset
    waitsome();
    sendlcd(0x1C6); //initial settings
    waitsome();
    
    sendlcd(0x1B9); //refset
    sendlcd(0x0);
    
    sendlcd(0x1b6); //Display control
    sendlcd(0x80);
    sendlcd(0x80);
    sendlcd(0x81);
    sendlcd(84);
    sendlcd(69);
    sendlcd(82);
    sendlcd(67);
    
    sendlcd(0x1B3); //Grey scale pos (B3,B4)
//    sendlcd(1);
    for (x=0; x<15; x++) {
	sendlcd(x*8);
    }

    sendlcd(0x1b5); //Gamma curve
    sendlcd(1);
    
    sendlcd(0x1bd); //Common driver out
    sendlcd(4); //0?
    
    sendlcd(0x1be); //power supply
    sendlcd(0x4);

    sendlcd(0x111); //sleep out
    waitsome();
    
    sendlcd(0x1BA);
    sendlcd(0xff);
    sendlcd(0x3);
    
    sendlcd(0x1b7); //Temp gradient
    for (x=0; x<14; x++) sendlcd(0); //semi-dummy
    
    sendlcd(0x103); //Booster pwr on
    waitsome();
    waitsome();
    waitsome();
    
    sendlcd(0x120); //inversion off
//    sendlcd(0x125); //contrast
//    sendlcd(110);
    
    sendlcd(0x129); //display on
    */
    sendlcd(0x120); //inversion off
    sendlcd(0x125); //contrast
    sendlcd(80);

}

#include "shagadelic.h"



//Initialize palette.
//0-128: some kind of hue-change, looks nice, wraps around.
//128+(n*16): 16 colors with the primary 3 bits of n being the primary
//   bits (rgb) being switched on or off, from intensity 0 to 16.
void initpal(char i) {
    int r,g,b,x,y;

    for (x=0; x<16; x++) {
        for (y=0; y<8; y++) {
	    pal[x+128+(y*16)]=0;
	    if (y&1) pal[x+128+(y*16)]|=(x<<8);
	    if (y&2) pal[x+128+(y*16)]|=(x<<4);
	    if (y&4) pal[x+128+(y*16)]|=(x);
	}
    }

    if (i==0) {
	for (x=0; x<128; x++) {
	    r=(isin(x*8)/4096)+8;
	    g=(isin(x*8+341)/4096)+8;
	    b=(isin(x*8+682)/4096)+8;
	    pal[x]=(r<<8)+(g<<4)+b;
	}
    } else if (i==1) {
	for (x=0; x<64; x++) {
	    r=(x>>2);
	    g=(x>>2);
	    b=15;
	    pal[x+64]=(r<<8)+(g<<4)+b;
	}
	for (x=0; x<64; x++) {
	    r=0;
	    g=0;
	    b=x>>2;
	    pal[x]=(r<<8)+(g<<4)+b;
	}
    } else if (i==2) {
	for (x=0; x<255; x++) {
	    r=header_data_cmap[x][0]>>4;
	    g=header_data_cmap[x][1]>>4;
	    b=header_data_cmap[x][2]>>4;	
	    pal[x]=(r<<8)+(g<<4)+b;
	}
    }
#ifdef INVERTED_LCD
    for (x=0; x<255; x++) pal[x]=0xfff-pal[x];
#endif
}

//Lookup dispram[] into the clut pal[] and send the data to the LCD as
//12-bit colors. Quite fast due to hardware-accellerated SPI (if enabled)
void senddisp() {
    int x;
    int o;
    sendlcd(0x12c);
    for (x=0; x<67*98; x+=2) {
	o=(pal[dispram[x]]<<12)+pal[dispram[x+1]];
	sendlcd((o>>16)&0xff);
	sendlcd((o>>8)&0xff);
	sendlcd(o&0xff);
    }
}

